@@ -1,4 +1,4 @@
-﻿# Copyright 2022 The Johns Hopkins University Applied Physics Laboratory
+﻿# Copyright 2016 The Johns Hopkins University Applied Physics Laboratory
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +15,9 @@
 import unittest
 from intern.resource.boss.resource import CollectionResource
 
-
 class TestCollectionResource(unittest.TestCase):
     def setUp(self):
-        self.coll = CollectionResource("foo")
+        self.coll = CollectionResource('foo')
 
     def test_not_valid_volume(self):
         self.assertFalse(self.coll.valid_volume())
@@ -27,4 +26,4 @@ class TestCollectionResource(unittest.TestCase):
         self.assertEqual(self.coll.name, self.coll.get_route())
 
     def test_get_list_route(self):
-        self.assertEqual("foo/", self.coll.get_list_route())
+        self.assertEqual('', self.coll.get_list_route())
