@@ -16,7 +16,6 @@ from abc import ABCMeta, abstractmethod
 from intern.resource.boss.resource import CoordinateFrameResource
 from requests import Request
 
-
 @six.add_metaclass(ABCMeta)
 class BaseVersion(object):
     """BaseVersion is the base class for all versioned interfaces to the
@@ -32,6 +31,25 @@ class BaseVersion(object):
         """
         raise NotImplemented
 
+    def convert_int_list_to_comma_sep_str(self, int_list):
+        """Convert list of ints to comma separated stringj.
+
+        Args:
+            int_list (list[int]): list of ints.
+
+        Returns:
+            (string): Example: [1, 7, 9] => '1, 7, 9'
+
+        """
+        if len(int_list) == 1:
+            return str(int_list[0])
+
+        str_list = []
+        for n in int_list:
+            str_list.append(str(n))
+
+        return (',').join(str_list)
+
     def convert_int_list_range_to_str(self, int_list):
         """Convert range in list of two ints to string representation.
 
@@ -128,9 +146,73 @@ class BaseVersion(object):
         return urlWithKey + '&value=' + str(value)
 
     def build_cutout_url(
-        self, resource, url_prefix, resolution, x_range, y_range, z_range, time_range):
+        self, resource, url_prefix, resolution, x_range, y_range, z_range, time_range=None, id_list=[], access_mode=None):
         """Build the url to access the cutout function of the Boss' volume service.
 
+        Args:
+            url_prefix (string): Do not end with a slash.  Example of expected value: https://api.theboss.io
+            resolution (int): 0 indicates native resolution.
+            x_range (list[int]): x range such as [10, 20] which means x>=10 and x<20.
+            y_range (list[int]): y range such as [10, 20] which means y>=10 and y<20.
+            z_range (list[int]): z range such as [10, 20] which means z>=10 and z<20.
+            time_range (optional [list[int]]): time range such as [30, 40] which means t>=30 and t<40.
+            id_list (optional [list[int]]): list of object ids to filter the cutout by.
+            access_mode (optional [Enum]): Identifies one of three cache access options:
+                cache = Will check both cache and for dirty keys
+                no_cache = Will skip cache check but check for dirty keys
+                raw = Will skip both the cache and dirty keys check
+                NOTE: No default here since when building a cutout url to create_cutout there is no need for access_mode
+
+        Returns:
+            (string): Full URL to access API.
+
+        Raises:
+            (RuntimeError): if *_range invalid.
+        """
+        base_url = self.build_url(resource, url_prefix, 'cutout', req_type='cutout')
+        x_rng_lst = self.convert_int_list_range_to_str(x_range)
+        y_rng_lst = self.convert_int_list_range_to_str(y_range)
+        z_rng_lst = self.convert_int_list_range_to_str(z_range)
+
+        url_with_params = (
+            base_url + '/' + str(resolution) + '/' + x_rng_lst + '/' + y_rng_lst +
+            '/' + z_rng_lst + '/')
+
+        if time_range is not None:
+            t_rng_lst = self.convert_int_list_range_to_str(time_range)
+            url_with_params += t_rng_lst + '/'
+
+        queryParamDict = {}
+        if len(id_list) > 0:
+            queryParamDict['filter'] = self.convert_int_list_to_comma_sep_str(id_list)
+    
+        # If creating a cutout, the url will not include the access_mode otherwise it will
+        if access_mode is not None:
+            queryParamDict['access-mode'] = access_mode.value 
+        """
+        TODO: LMR
+        The following could be done using urlib.urlencode(url_with_params += '?' + urllib.parse.urlencode(queryParamDict,safe=",")),
+        however urllib's python2 version of this function does not take in the 'safe' parameter and thus we can not use the 
+        function interchangable for python2/3. In order to keep our python2/3 compatability, we do not use urllib. 
+        """
+        if queryParamDict:
+            # The first time include '?'
+            url_with_params += '?'
+            for k, v in queryParamDict.items():
+                # if this is the first run through, last char in str will be ?, so don't include '&'
+                if url_with_params[len(url_with_params)-1] == '?':
+                    pass
+                # otherwise use '&'
+                else:
+                    url_with_params += '&'
+                # Add key and value members
+                url_with_params += '{}={}'.format(k,v)
+        return url_with_params
+
+    def build_cutout_to_black_url(
+        self, resource, url_prefix, resolution, x_range, y_range, z_range, time_range=None):
+        """Build the url to access the cutout_to_black function of the Boss' volume service.
+        TODO: Maybe delete this function and add a base_url parameter to build_cutout_url?
         Args:
             url_prefix (string): Do not end with a slash.  Example of expected value: https://api.theboss.io
             resolution (int): 0 indicates native resolution.
@@ -145,20 +227,20 @@ class BaseVersion(object):
         Raises:
             (RuntimeError): if *_range invalid.
         """
-        baseUrl = self.build_url(resource, url_prefix, 'cutout', req_type='cutout')
+        base_url = self.build_url(resource, url_prefix, 'cutout/to_black', req_type='cutout')
         x_rng_lst = self.convert_int_list_range_to_str(x_range)
         y_rng_lst = self.convert_int_list_range_to_str(y_range)
         z_rng_lst = self.convert_int_list_range_to_str(z_range)
 
-        urlWithParams = (
-            baseUrl + '/' + str(resolution) + '/' + x_rng_lst + '/' + y_rng_lst +
+        url_with_params = (
+            base_url + '/' + str(resolution) + '/' + x_rng_lst + '/' + y_rng_lst +
             '/' + z_rng_lst + '/')
-        if time_range is None:
-            return urlWithParams
 
-        t_rng_lst = self.convert_int_list_range_to_str(time_range)
-        urlWithTime = urlWithParams + t_rng_lst + '/'
-        return urlWithTime
+        if time_range is not None:
+            t_rng_lst = self.convert_int_list_range_to_str(time_range)
+            url_with_params += t_rng_lst + '/'
+        
+        return url_with_params
 
     def get_request(self, resource, method, content, url_prefix, token, proj_list_req=False, json=None, data=None):
         """Create a request for accessing the Boss' data model services.
@@ -227,7 +309,7 @@ class BaseVersion(object):
 
     def get_cutout_request(
         self, resource, method, content, url_prefix, token,
-        resolution, x_range, y_range, z_range, time_range, numpyVolume=None):
+        resolution, x_range, y_range, z_range, time_range,  numpyVolume=None, id_list=[], access_mode=None,):
 
         """Create a request for working with cutouts (part of the Boss' volume service).
 
@@ -241,8 +323,13 @@ class BaseVersion(object):
             x_range (list[int]): x range such as [10, 20] which means x>=10 and x<20.
             y_range (list[int]): y range such as [10, 20] which means y>=10 and y<20.
             z_range (list[int]): z range such as [10, 20] which means z>=10 and z<20.
-            time_range (optional [list[int]]): time range such as [30, 40] which means t>=30 and t<40.
-            numpyVolume (numpy array): The data volume encoded in a numpy array.
+            time_range (list[int]): time range such as [30, 40] which means t>=30 and t<40.
+            numpyVolume (optional numpy array): The data volume encoded in a numpy array.
+            id_list (optional [list[int]]): list of object ids to filter the cutout by.
+            access_mode (optional [Enum]): Identifies one of three cache access options:
+                cache = Will check both cache and for dirty keys
+                no_cache = Will skip cache check but check for dirty keys
+                raw = Will skip both the cache and dirty keys check
 
         Returns:
             (requests.Request): A newly constructed Request object.
@@ -251,9 +338,38 @@ class BaseVersion(object):
             RuntimeError if url_prefix is None or an empty string.
         """
         url = self.build_cutout_url(
-            resource, url_prefix, resolution, x_range, y_range, z_range, time_range)
+            resource, url_prefix, resolution, x_range, y_range, z_range,  time_range, id_list, access_mode)
         headers = self.get_headers(content, token)
         return Request(method, url, headers = headers, data = numpyVolume)
+    
+    def get_cutout_to_black_request(
+        self, resource, method, content, url_prefix, token,
+        resolution, x_range, y_range, z_range, time_range):
+
+        """Create a request for working with cutout_to_black (part of the Boss' volume service).
+
+        Args:
+            resource (intern.resource.boss.BossResource): Resource to perform operation on.
+            method (string): HTTP verb such as 'GET'.
+            content (string): HTTP Content-Type such as 'application/json'.
+            url_prefix (string): protocol + initial portion of URL such as https://api.theboss.io  Do not end with a forward slash.
+            token (string): Django Rest Framework token for auth.
+            resolution (int): 0 indicates native resolution.
+            x_range (list[int]): x range such as [10, 20] which means x>=10 and x<20.
+            y_range (list[int]): y range such as [10, 20] which means y>=10 and y<20.
+            z_range (list[int]): z range such as [10, 20] which means z>=10 and z<20.
+            time_range (list[int]): time range such as [30, 40] which means t>=30 and t<40.
+
+        Returns:
+            (requests.Request): A newly constructed Request object.
+
+        Raises:
+            RuntimeError if url_prefix is None or an empty string.
+        """
+        url = self.build_cutout_to_black_url(
+            resource, url_prefix, resolution, x_range, y_range, z_range,  time_range)
+        headers = self.get_headers(content, token)
+        return Request(method, url, headers = headers)
 
     def get_group_request(self, method, content, url_prefix, token, name=None):
         """Get a request for getting group information.
@@ -438,3 +554,122 @@ class BaseVersion(object):
 
         headers = self.get_headers(content, token)
         return Request(method, url, headers=headers, json=data)
+
+    def get_reserve_request(
+            self, resource, method, content, url_prefix, token, num_ids):
+        """
+
+        Args:
+            resource (intern.resource.boss.BossResource): Resource to perform operation on.
+            method (string): HTTP verb such as 'GET'.
+            content (string): HTTP Content-Type such as 'application/json'.
+            url_prefix (string): protocol + initial portion of URL such as https://api.theboss.io  Do not end with a forward slash.
+            token (string): Django Rest Framework token for auth.
+            num_ids (int): Number of ids to reserve.
+
+        Returns:
+            (requests.Request): A newly constructed Request object.
+
+        Raises:
+            RuntimeError if url_prefix is None or an empty string.
+        """
+        if url_prefix is None or url_prefix == '':
+            raise RuntimeError('url_prefix required.')
+
+        url = (url_prefix + '/' + self.version + '/reserve/' +
+               resource.get_reserve_route() + '/{}'.format(num_ids))
+        headers = self.get_headers(content, token)
+        return Request(method, url, headers=headers)
+
+    def get_bounding_box_request(
+            self, resource, method, content, url_prefix, token, resolution,
+            id, bb_type):
+        """
+
+        Args:
+            resource (intern.resource.boss.BossResource): Resource to perform operation on.
+            method (string): HTTP verb such as 'GET'.
+            content (string): HTTP Content-Type such as 'application/json'.
+            url_prefix (string): protocol + initial portion of URL such as https://api.theboss.io  Do not end with a forward slash.
+            token (string): Django Rest Framework token for auth.
+            resolution (int): 0 = default resolution.
+            id (int): Annotation object id.
+            bb_type (string): 'loose' | 'tight'.
+
+        Returns:
+            (requests.Request): A newly constructed Request object.
+
+        Raises:
+            RuntimeError if url_prefix is None or an empty string.
+        """
+        if url_prefix is None or url_prefix == '':
+            raise RuntimeError('url_prefix required.')
+
+        url = (url_prefix + '/' + self.version + '/boundingbox/' +
+               resource.get_cutout_route() + '/{}/{}/?type={}'.format(
+            resolution, id, bb_type))
+        headers = self.get_headers(content, token)
+        return Request(method, url, headers=headers)
+
+    def build_ids_url(
+            self, resource, url_prefix, resolution,
+            x_range, y_range, z_range, time_range=None):
+        """Build the url to access the ids function of the Boss' volume service.
+
+        Args:
+            resource (intern.resource.boss.BossResource): Resource to perform operation on.
+            url_prefix (string): Do not end with a slash.  Example of expected value: https://api.theboss.io
+            resolution (int): 0 indicates native resolution.
+            x_range (list[int]): x range such as [10, 20] which means x>=10 and x<20.
+            y_range (list[int]): y range such as [10, 20] which means y>=10 and y<20.
+            z_range (list[int]): z range such as [10, 20] which means z>=10 and z<20.
+            time_range (optional [list[int]]): time range such as [30, 40] which means t>=30 and t<40.
+
+        Returns:
+            (string): Full URL to access API.
+
+        Raises:
+            (RuntimeError): if *_range invalid.
+        """
+        base_url = self.build_url(resource, url_prefix, 'ids', req_type='cutout')
+        x_rng_lst = self.convert_int_list_range_to_str(x_range)
+        y_rng_lst = self.convert_int_list_range_to_str(y_range)
+        z_rng_lst = self.convert_int_list_range_to_str(z_range)
+
+        url_with_params = (
+            base_url + '/' + str(resolution) + '/' + x_rng_lst + '/' + y_rng_lst +
+            '/' + z_rng_lst + '/')
+
+        if time_range is not None:
+            t_rng_lst = self.convert_int_list_range_to_str(time_range)
+            url_with_params += t_rng_lst + '/'
+
+        return url_with_params
+
+    def get_ids_request(
+            self, resource, method, content, url_prefix, token,
+            resolution, x_range, y_range, z_range, time_range):
+        """Create a request for getting ids in a region (part of the Boss' volume service).
+
+        Args:
+            resource (intern.resource.boss.BossResource): Resource to perform operation on.
+            method (string): HTTP verb such as 'GET'.
+            content (string): HTTP Content-Type such as 'application/json'.
+            url_prefix (string): protocol + initial portion of URL such as https://api.theboss.io  Do not end with a forward slash.
+            token (string): Django Rest Framework token for auth.
+            resolution (int): 0 indicates native resolution.
+            x_range (list[int]): x range such as [10, 20] which means x>=10 and x<20.
+            y_range (list[int]): y range such as [10, 20] which means y>=10 and y<20.
+            z_range (list[int]): z range such as [10, 20] which means z>=10 and z<20.
+            time_range (list[int]): time range such as [30, 40] which means t>=30 and t<40.
+
+        Returns:
+            (requests.Request): A newly constructed Request object.
+
+        Raises:
+            RuntimeError if url_prefix is None or an empty string.
+        """
+        url = self.build_ids_url(
+            resource, url_prefix, resolution, x_range, y_range, z_range, time_range)
+        headers = self.get_headers(content, token)
+        return Request(method, url, headers=headers)
