@@ -184,7 +184,7 @@ class VolumeService_1(BaseVersion):
 
     def get_cutout(
             self, resource, resolution, x_range, y_range, z_range, time_range, id_list,
-            url_prefix, auth, session, send_opts, access_mode=CacheMode.no_cache, parallel: bool = True, **kwargs
+            url_prefix, auth, session, send_opts, access_mode=CacheMode.no_cache, parallel=True, **kwargs
         ):
         """
         Upload a cutout to the Boss data store.
@@ -207,8 +207,9 @@ class VolumeService_1(BaseVersion):
                 no_cache = Will skip cache check but check for dirty keys
                 raw = Will skip both the cache and dirty keys check
             chunk_size (optional Tuple[int, int, int]): The chunk size to request
-            parallel (bool: True): Whether downloads should be parallelized using multiprocessing
-
+            parallel (Union[int, bool]: True): Whether downloads should be parallelized using 
+                multiprocessing. If set to True, will use all available CPUs. If set to False,
+                will use only one CPU. If set to an integer, will spawn that number of threads.
 
         Returns:
             (numpy.array): A 3D or 4D numpy matrix in ZXY(time) order.
@@ -216,7 +217,17 @@ class VolumeService_1(BaseVersion):
         Raises:
             requests.HTTPError
         """
-        chunk_size = kwargs.pop("chunk_size", (512, 512, 16 * 8))
+        if parallel:
+            # Parallel downloads are faster with a smaller chunk size but can easily overwhelm
+            # the endpoint if its too small. Therefore from empirical testing (512, 512, 96) 
+            # USUALLY is the fastest. There is some variabiity on number of threads. 
+            chunk_size = kwargs.pop("chunk_size", (512, 512, 16 * 6))
+        else:
+            # Single thread downloads are faster with a large chunk size, but can't surpass 
+            # 500 MB limit. To stay within 500 MB constraint with 64-bit data, we chose a 
+            # chunk size of (512, 512, 192) which is about 402 MB. 
+            chunk_size = kwargs.pop("chunk_size", (512, 512, 16 * 12))
+        
         # TODO: magic number
         chunk_limit = (chunk_size[0] * chunk_size[1] * chunk_size[2]) * 1.2
 
@@ -251,15 +262,21 @@ class VolumeService_1(BaseVersion):
             ), dtype=resource.datatype)
 
             if parallel:
-                pool = multiprocessing.Pool(processes=parallel if isinstance(parallel, int) and parallel > 0 else multiprocessing.cpu_count())
-                chunks = pool.starmap(self.get_cutout, [
-                    (
-                        resource, resolution, b[0], b[1], b[2],
-                        time_range, id_list, url_prefix, auth, session, send_opts,
-                        access_mode
-                        # TODO: kwargs
-                    )
-                 for b in blocks])
+                if type(parallel) == bool:
+                    parallel = multiprocessing.cpu_count()
+                elif parallel > 0:
+                    parallel = int(parallel)
+                else:
+                    raise ValueError("Parallel must be greater than 0.")
+                with multiprocessing.Pool(processes=parallel) as pool:
+                    chunks = pool.starmap(self.get_cutout, [
+                        (
+                            resource, resolution, b[0], b[1], b[2],
+                            time_range, id_list, url_prefix, auth, session, send_opts,
+                            access_mode
+                            # TODO: kwargs
+                        )
+                     for b in blocks])
 
                 for b, data in zip(blocks, chunks):
                     result[
@@ -296,7 +313,7 @@ class VolumeService_1(BaseVersion):
 
         if resp.status_code == 200:
             raw_data = blosc.decompress(resp.content)
-            data_mat = np.fromstring(raw_data, dtype=resource.datatype)
+            data_mat = np.frombuffer(raw_data, dtype=resource.datatype)
 
             if time_range:
                 # Reshape including time
