@@ -57,8 +57,8 @@ def snap_to_cube(q_start, q_stop, chunk_depth=16, q_index=1):
 def block_compute(x_start, x_stop,
                   y_start, y_stop,
                   z_start, z_stop,
-                  origin=(0, 0, 0),
-                  block_size=(512, 512, 16)):
+                  origin=(0, 0, 1),
+                  block_size=(256, 256, 16)):
     """
     Get bounding box coordinates (in 3D) of small cutouts to request in
     order to reconstitute a larger cutout.
@@ -78,7 +78,7 @@ def block_compute(x_start, x_stop,
     # x
     x_bounds = range(origin[0], x_stop + block_size[0], block_size[0])
     x_bounds = [x for x in x_bounds if (x > x_start and x < x_stop)]
-    if len(x_bounds) == 0:
+    if len(x_bounds) is 0:
         x_slices = [(x_start, x_stop)]
     else:
         x_slices = []
@@ -88,9 +88,9 @@ def block_compute(x_start, x_stop,
         x_slices.append((x_bounds[-1], x_stop))
 
     # y
-    y_bounds = range(origin[1], y_stop + block_size[1], block_size[1])
+    y_bounds = range(origin[1], x_stop + block_size[1], block_size[1])
     y_bounds = [y for y in y_bounds if (y > y_start and y < y_stop)]
-    if len(y_bounds) == 0:
+    if len(y_bounds) is 0:
         y_slices = [(y_start, y_stop)]
     else:
         y_slices = []
@@ -102,7 +102,7 @@ def block_compute(x_start, x_stop,
     # z
     z_bounds = range(origin[2], z_stop + block_size[2], block_size[2])
     z_bounds = [z for z in z_bounds if (z > z_start and z < z_stop)]
-    if len(z_bounds) == 0:
+    if len(z_bounds) is 0:
         z_slices = [(z_start, z_stop)]
     else:
         z_slices = []
